function [out1,out2,out3,out4,out5] = fadaSingleRun(X,T,NumS,NumA,Spos,Cpos,del,indep)

    % fadaSingleRun performs Fourier-based demixing algorithm of the input data
    % only once
    %
    %  Input: X     = Matrix of the original data.
    %         T     = Number of time samples per signal
    %         NumS  = Number of primitives (components) to learn
    %         NumA  = Number of Fourier harmonics to approximate original signal
    %         Spos  = positivity of the primitives 0 = unconstrain
    %                                             1 = positive
    %         Cpos  = positivity of the combination coefficients
    %                                             0 = unconstrained
    %                                             1 = positive
    %         del   = modelling of the delays     0 = non-shiftable primitives
    %                                             1 = shiftable primitives
    %         indep = independence of the sources  'ica' = independent sources
    %                                              'unc' = uncorrelated sources
    %                                              'lsq' = Least-square approx.
    %
    %   Output:
    %          S    = identified primitives
    %          W    = identified combination coefficients
    %         Tau   = identified delays
    %         Rec   = reconstructed data
    %          p    = variance accounted for
    % 
    % Copyright (C) Enrico Chiovetto, 2014, enrico.chiovetto@gmail.com.
    % 
    % This program is free software: you can redistribute it and/or modify
    % it under the terms of the GNU General Public License as published by
    % the Free Software Foundation, either version 3 of the License, or
    % (at your option) any later version.
    % 
    % This program is distributed in the hope that it will be useful,
    % but WITHOUT ANY WARRANTY; without even the implied warranty of
    % MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    % GNU General Public License for more details.
    % 
    % You should have received a copy of the GNU General Public License
    % along with this program.  If not, see <http://www.gnu.org/licenses/>.
    %
    % If you use this software for your research, please refer to the paper
    % entitled " ", 
    % by Chiovetto E, d'Avella A and Giese, MA.

    %---------- Fix the parameters
    maxiter=50; % Maximum number of iterations
    miniter=10;   % Number of iterations after which the algorithm stops if no 
                 % improvement higher than minimprov reached during the last
                 % miniter iteractions

    %---------- Check the data

    if and(and(Spos==1,Cpos==1),~(isempty(find(X<0, 1))))
       error('The imput matrix must contain only non-negative data.');
    end
    [K,MT] = size(X); % K = Number of dof, T = Number of time points
                      % M = Number of Trials. dof=degrees of freedom              
    T0 = 2*pi;
    if MT==T
        M=K;
        K=1;
        X=reshape(X',1,numel(X));
    else
        M=MT/T;
    end
    dt = T0/T;
    ts = 0:dt:(T0-dt);
    iter=1;
    best_p =-inf;

    %---------- Check the boudaries of maxiter and miniter
    if maxiter-miniter<miniter
        maxiter = maxiter+miniter;
    end

    %---------- Compute the Fourier coefficients of the data
    for k=1:K
        RX=reshape(X(k,:),T,M)';
        [gamma]= HarmonicFourierSeries(RX,floor(T/2));
        A=gamma(:,2:2:(2*floor(T/2)+1));
        B=gamma(:,3:2:(2*floor(T/2)+1));
        F(k,:,:)=[gamma(:,1)/2 (A-1i*B)/2];
        modFsq=abs(squeezeOne(F(k,:,:))).^2;
    end
    disp(['Number of Primitives = ',num2str(NumS)]);
    if or(isempty(NumA),NumA>floor(T/2))
        NumA=floor(T/2)-1;
    end
    disp(['Number of Harmonics = ',num2str(NumA)]);

    %--------------------------------------------------------------------------
    % First part of the algorithm. Compute ICA or NMF to factorize the absolute
    % values of the Fourier coefficients of the signals.
    %--------------------------------------------------------------------------
    modS=zeros(K,NumS,NumA+1);
    if Spos==0
        F=[];
        H=[];
        W=[];
        for k=1:K
            RX=reshape(X(k,:),T,M)';
            [gamma]=HarmonicFourierSeries(RX,NumA);
            A=gamma(:,2:2:(2*NumA+1));
            B=gamma(:,3:2:(2*NumA+1));
            F(k,:,:)=[gamma(:,1)/2 (A-1i*B)/2];
            modFsq=abs(squeezeOne(F(k,:,:))).^2;
            PAR.Sprior='exponential';
            PAR.Aprior='positive';
            PAR.solver = 'variational';
            PAR.optimizer='conjgrad';
            PAR.sources=  NumS;
            PAR.max_ite=500;
            PAR.S_tol=10^(-200);
            PAR.tol=10^(-80);
            PAR.S_max_ite=100;
            if strcmp(indep,'ica')
                [H(k,:,:),W(k,:,:)]=icaMF(modFsq/max(max(abs(modFsq))),PAR);
            else
                if or(strcmp(indep,'lsq'),strcmp(indep,'unc'))
                [W(k,:,:),H(k,:,:)] = NNMF(modFsq,NumS);
                end
            end
            modS(k,:,:)=sqrt(squeezeOne(H(k,:,:)));
            phaseS(k,:,:)= random('unif',-pi,pi,NumS,NumA);
            W(k,:,:)=realsqrt(abs(squeezeOne(W(k,:,:))));
        end
        S=ones(NumS,K,T);
        Tau = zeros(M,NumS);
        for j=1:M
            Weights(j,:)=mean(squeezeOne(W(:,j,:)),1);
        end
    else
        RX=[];
        for j=1:M
            RX=[RX; reshape(X(:,(j-1)*T+1:j*T)',1,T*K)];
        end
        [Weights,h]=NNMF(abs(RX),NumS);
        for l=1:K
            [gamma]=HarmonicFourierSeries(h(:,(l-1)*T+1:l*T),NumA);
            A=gamma(:,2:2:(2*NumA+1));
            B=gamma(:,3:2:(2*NumA+1));
            modS(l,:,:)=abs([gamma(:,1)/2 (A-1i*B)/2]);
            phaseS(l,:,:)=angle([(A-1i*B)/2]);
        end
        Tau = zeros(M,NumS);
        S=ones(NumS,K,T);
    end
    p(1)=0;

    %--------------------------------------------------------------------------
    % Second main part of the algorithm. Update of Fourier coefficients,
    % weights and delays.
    %--------------------------------------------------------------------------
    RX = reshape(X,K,T,M);
    while (iter<=maxiter && iter>miniter) || (iter<=miniter)
        disp(['Iteration ',num2str(iter)]);
        %---------- Update primitives
        disp('Update primitives');
        for k=1:K
            disp(['dof ', num2str(k),' out of ',num2str(K)]);
            lb=-pi*ones(1,NumS*NumA);
            ub=pi*ones(1,NumS*NumA);
            warning off;
            if  and(Spos==0,strcmp(indep,'lsq'))
                D = diag(repmat(ones(1,NumA+1),1,NumS));
                options = optimset('Diagnostic','off','MaxIter',100000,...
                    'MaxFunEvals',50000,'Display','off');
                [mod]=fminunc(@(x)FunModS(D*x', Weights, Tau, phaseS, F,k),...
                    reshape(squeezeOne(modS(k,:,:))',1,NumS*(NumA+1)),options);
                modS(k,:,:)=reshape(mod,NumA+1,NumS)';
            end
            if Spos==1
                D = diag(repmat(ones(1,NumA+1),1,NumS));
                options = optimset('TolCon',1e-10,'TolX',1e-6,'TolFun',1e-6,...
                    'Diagnostic','off','MaxIter',3000,'MaxFunEvals',50000,...
                    'Display','off','GradConstr','off');
                [mod]=fmincon(@(x)FunModS(D*x', Weights, Tau, phaseS, F,k),...
                    reshape(squeezeOne(modS(k,:,:))',1,NumS*(NumA+1)),[],[],...
                    [],[],zeros(NumS*(NumA+1)),...
                    max(max(abs(X))),@(x)nonlinconModS(D*x', phaseS, ts, k),...
                    options);
                modS(k,:,:)=reshape(mod,NumA+1,NumS)';
            end
            if  Spos==0
                options = optimset('Diagnostic','off','MaxIter',1000,...
                    'MaxFunEvals',50000,'Display','off','algorithm',...
                    'trust-region-reflective');
                [phase]=lsqnonlin(@(x)FunPhaseS(x, Weights, Tau, modS, F, k),...
                    reshape(squeezeOne(phaseS(k,:,:))',1,NumS*(NumA)),...
                    [],[],options);
            end
            if  Spos==1
                options = optimset('TolCon',1e-10,'TolX',1e-6,'TolFun',1e-6,...
                    'Diagnostic','off','MaxIter',3000,'MaxFunEvals',50000,...
                    'Display','off','GradConstr','off');
                [phase]=fmincon(@(x)FunPhaseS(x, Weights, Tau, modS, F, k),...
                    reshape(squeezeOne(phaseS(k,:,:))',1,NumS*(NumA)),[],[],...
                    [],[],lb,ub,@(x)nonlinconPhaseS(x, modS, ts, k),options);
            end
            phaseS(k,:,:)=real(reshape(phase,NumA,NumS)');
            temp=squeezeOne(phaseS(k,:,:));
            Ss=squeezeOne(modS(k,:,2:NumA+1)).*exp(1i*temp(:,1:NumA));
            temp1=squeezeOne(modS(k,:,:));
            Stemp=repmat(temp1(:,1),1,T);
            for l=1:1:NumS
                for m=1:1:NumA
                    Stemp(l,:)=Stemp(l,:)+conj(Ss(l,m))*exp(-2i*pi*m*ts/T0)+...
                        +Ss(l,m)*exp(2i*pi*m*ts/T0);
                end
            end
            S(:,k,:)=Stemp;
        end

        %---------- Update weights and delays
        disp('Update weights (and delays)');

        for m=1:M,
            Data = squeezeOne(RX(:,:,m));
            W = Weights(m,:);
            D = Tau(m,:)*T/T0;
            [Weights(m,:),Tau(m,:),Rec(:,:,m)] = weightsDelayEstimation...
                (Data,S,W,D,round(T)*ones(NumS,1),Cpos,del);
            Tau(m,:) = Tau(m,:)*T0/T;
        end
      

        %---------- Reconstruct original data
        if K == 1
            Rec_final = squeezeOne(Rec(1,:,:))';
        else
            Rec_final = zeros(K,M*T);
            for m = 1:M
                 Rec_final(:,(m-1)*T+1:m*T) = squeezeOne(Rec(:,:,m));
            end
        end

        
        %---------- Compute similarity between original and reconstructed data
        if K == 1
            p(iter) = 1-(norm(Rec_final-reshape(X,T,M)','fro')^2)/norm...
                (reshape(X,T,M)'-repmat(mean(reshape(X,T,M),1)',1,size...
                (reshape(X,T,M)',2)),'fro')^2;
        else
            p(iter) = 1-(norm(Rec_final-X,'fro')^2)/norm...
                (X-repmat(mean(X,2),1,size(X,2)),'fro')^2;
        end
        disp(['p = ',num2str(p(iter))]);

        %---------- Update outputs
        if or(p(iter)>best_p && iter>1, iter==maxiter)
            best_p = p(iter);
            if K==1
                 out1 = squeeze(S(:,1,:));
            else
                 out1 = permute(S,[2 1 3]);
            end
            out2 = Weights;
            out3 = Tau*T/T0;
            out4 = Rec_final;
            out5 = p(iter);
        end
        iter=iter+1;

    end