function[out1,out2,out3,out4,out5,out6] = fadaSingleRunNM3F(X,T,NumS,NumW,NumA,del)

% fadaSingleRunNM3F performs Fourier-based demixing algorithm of the input
% data only once. This function is specifically designed for the
% identification od space-by-time primitives, with non-negativity
% constraints on all the parameters as imposed in the model by Delis and
% colleagues.
%
%  Inputs: X    = Matrix of the original data
%                 X has to be a matrix with time signals of the
%                 same trial piled vertically and signals of different
%                 trials concatenated horizontally. X will thus have size
%                 (NumM) x (NumT x T).  NumT is the number of trials, NumM 
%                 the number of degrees of freedom recorded in each trial,
%                 for instance the number of muscles.All signals need to be
%                 standardized in time in a pre-processing phase.
%
%         T     = Number of time samples per signal
%         NumS  = Number of primitives to learn
%         NumA  = Number of Fourier harmonics to approximate the original
%         signals
%         Spos  = positivity of the primitives 0 = unconstrained
%                                             1 = positive
%         Cpos  = positivity of the combination coefficients
%                                             0 = unconstrained
%                                             1 = positive
%         del   = modelling of the delays     0 = non-shiftable primitives
%                                             1 = time shiftable primitives
%         NumRuns = number of runs
%         indep = independence of the sources  'ica' = independent sources
%                                              'nmf' = uncorrelated sources
%                                              'lsq' = Least-square approx.
%
%   Outputs:
%          S    = identified primitives
%          W    = identified combination coefficients
%         Tau   = identified delays
%         Rec   = reconstructed data
%          p    = variance accounted for
%
% Copyright (C) Enrico Chiovetto, 2014, enrico.chiovetto@gmail.com.
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% If you use this software for your research, please refer to the paper
% entitled " ",
% by Chiovetto E, d'Avella A and Giese, MA.

%---------- Fix the parameters
maxiter=50; % Maximum number of iterations
miniter=10;   % Number of iterations after which the algorithm stops if no
% improvement higher than minimprov reached during the last
% miniter iteractions

if ~(isempty(find(X<0, 1)))
    error('The imput matrix must contain only non-negative data.');
end
[K,MT] = size(X); % K = Number of Muscles, T Number of Time points, M Number of Trials
M      = MT/T;
T0 = 2*pi;
dt = (T0)/T;
ts = 0:dt:(T0-dt);
iter=1;
best_p =-inf;

%---------- Check the boudaries of maxiter and miniter
if maxiter-miniter<miniter
    maxiter = maxiter+miniter;
end

%---------- Compute the Fourier coefficients of the data
for k=1:K
    RX=reshape(X(k,:),T,M)';
    [gamma]=HarmonicFourierSeries(RX,floor(T/2));
    A=gamma(:,2:2:(2*floor(T/2)+1));
    B=gamma(:,3:2:(2*floor(T/2)+1));
    F(k,:,:)=[gamma(:,1)/2 (A-1i*B)/2];
    modFsq=abs(squeezeOne(F(k,:,:))).^2;
    indicator(k,:)=mean(abs(squeezeOne(F(k,:,:))).^2,1);
end
if or(isempty(NumA),NumA>floor(T/2))
    NumA=floor(T/2);
end
disp(['Number of tempooal primitives = ',num2str(NumS)]);
disp(['Number of spatial primitives  = ',num2str(NumW)]);
disp(['Number of Harmonics            = ',num2str(NumA)]);

%--------------------------------------------------------------------------
% First part of the algorithm. Compute ICA or NMF to factorize the absolute
% values of the Fourier coefficients of the signals.
%--------------------------------------------------------------------------

modS=zeros(1,NumS,NumA+1);
RX=[];
for j=1:M
    RX=[RX; X(:,(j-1)*T+1:j*T)];
end
[~,h_t]=NNMF(abs(RX),NumS);
[gamma]=HarmonicFourierSeries(h_t,NumA);
A=gamma(:,2:2:(2*NumA+1));
B=gamma(:,3:2:(2*NumA+1));
aux=abs([gamma(:,1)/2 (A-1i*B)/2]);
for l=1:1:NumS
    modS(1,l,:)=aux(l,:);
end
aux=angle([(A-1i*B)/2]);
for l=1:1:NumS
    phaseS(1,l,:)=aux(l,:);
end
Tau = 0*T/2*randn(M,NumS*NumW);
S=ones(NumS,K,T);
SS=ones(NumS*NumW,K,T);
% [Weights_s,h_s]=NNMF(abs(RX)',NumW,200);
Weights=max(max(X))*rand(M,NumS*NumW);
[~,C]=NNMF(X',NumW,200);C=C';

%--------------------------------------------------------------------------
% Second main part of the algorithm. Update of Fourier coefficients,
% weights and delays.
%--------------------------------------------------------------------------

RX = reshape(X,K,T,M);
while (iter<=maxiter) || (iter>miniter && iter<=maxiter)
    disp(['Iteration ',num2str(iter)]);
    %---------- Update primitives
    disp('Update prmitives');
    lb=-pi*ones(1,NumS*NumW*NumA);
    ub=pi*ones(1,NumS*NumW*NumA);
    warning off;
    options = optimset('TolCon',1e-10,'TolX',1e-6,'TolFun',1e-6,'Diagnostic','off','MaxIter',3000,'MaxFunEvals',50000,'Display','off','GradConstr','off');
    [mod]=fmincon(@(x)FunModSNM3F(x, Weights, Tau, phaseS, F, C),reshape(squeezeOne(modS(1,:,:))',1,NumS*(NumA+1)),[],[],[],[],zeros(NumS*(NumA+1)),...
        max(max(abs(X))),@(x)nonlinconModSNM3F(x, phaseS, ts),options);
    modS(1,:,:)=reshape(mod,NumA+1,NumS)';
    options = optimset('TolCon',1e-10,'TolX',1e-6,'TolFun',1e-6,'Diagnostic','off','MaxIter',3000,'MaxFunEvals',50000,'Display','off','GradConstr','off');
    [phase]=fmincon(@(x)FunPhaseSNM3F(x, Weights, Tau, modS, F, C),reshape(squeezeOne(phaseS(1,:,:))',1,NumS*(NumA)),[],[],[],[],lb,ub,@(x)nonlinconPhaseSNM3F(x, modS, ts),options);
    phaseS(1,:,:)=real(reshape(phase,NumA,NumS)');
    
    temp=squeezeOne(phaseS(1,:,:));
    Ss=squeezeOne(modS(1,:,2:NumA+1)).*exp(1i*temp(:,1:NumA));
    temp1=squeezeOne(modS(1,:,:));
    Stemp=repmat(temp1(:,1),1,T);
    for l=1:1:NumS
        for m=1:1:NumA
            Stemp(l,:)=Stemp(l,:)+conj(Ss(l,m))*exp(-1i*m*ts)+Ss(l,m)*exp(1i*m*ts);
        end
    end
    for k=1:NumS
        S(k,:,:)=repmat(Stemp(k,:),K,1);
    end
    for k=1:1:NumS
        for j=1:NumW
            SS((k-1)*NumW+j,:,:)=squeezeOne(S(k,:,:)).*repmat(C(:,j),1,T);
        end
    end
    
    %---------- Update weights and delays
    disp('Update weights (and delays)');
    
    R=[];
    for m=1:M,
        Data = squeezeOne(RX(:,:,m));
        W = Weights(m,:);
        D = Tau(m,:)*T/T0;
        [Weights(m,:),Tau(m,:),Rec(:,:,m)] = weightsDelayEstimation(Data,SS,W,D,T*ones(NumS*NumW,1),1,del);
        R=[R ,Rec(:,:,m)];
        Tau(m,:) = Tau(m,:)*T0/T;
    end
    
    %---------- Reconstruct original data
    if K == 1
        Rec_final = squeezeOne(Rec(1,:,:))';
    else
        Rec_final = zeros(K,M*T);
        for m = 1:M
            Rec_final(:,(m-1)*T+1:m*T) = squeezeOne(Rec(:,:,m));
        end
    end
    
    %---------- Compute similarity between original and reconstructed data
    if K == 1
        p(iter) = 1-(norm(Rec_final-reshape(X,T,M)','fro')^2)/norm...
            (reshape(X,T,M)'-repmat(mean(reshape(X,T,M),1)',1,size...
            (reshape(X,T,M)',2)),'fro')^2;
    else
        p(iter) = 1-(norm(Rec_final-X,'fro')^2)/norm...
            (X-repmat(mean(X,2),1,size(X,2)),'fro')^2;
    end
    disp(['p = ',num2str(p(iter))]);

    %---------- Update outputs
    if or(p(iter)>best_p && iter>1, iter==maxiter)
        best_p = p(iter);
        out1 = Stemp;
        out2 = C;
        out3 = Weights;
        out4 = Tau*T/T0;
        out5 = Rec_final;
        out6 = p(iter);
    end
    iter=iter+1;
    
end