function [S, Ss, W, Tau, Rec, p] = fada_NM3F(X,T,NumS,NumW,NumH,del,NumRuns)

% fada_NMF3F performs Fourier-based demixing of the input data and provide
% a tri-dimentional NMF factorization. This function works only for 
% non-negative data.
%
%  Inputs: X    = Matrix of the original data.
%                 X has to be a matrix with time signals of the 
%                 same trial piled vertically and signals of different  
%                 trials concatenated horizontally. X will thus have size 
%                 (Ndof) x (Ntr x T).  Ntr is the number of trials, Ndof 
%                 the number of degrees of freedom recorded in each trial,
%                 for instance the number of muscles).
%                 All signals need to be standardized in time in a 
%                 pre-processing phase.
%                   
%         T     = Number of time samples per signal
%         NumS  = Number of temporal primitives  
%         NumW  = Number of spatiL primitives  
%         NumH  = Number of Fourier Harmonics to approximate the original
%         signals 
%         Spos  = positivity of the primitives 0 = unconstrained
%                                             1 = positive
%         del   = modelling of the delays     0 = non-shiftable primitives
%                                             1 = time shiftable primitives
%         NumRuns = number of runs   
% 
%   Outputs:
%          S    = identified temporal primitives 
%          Ss   = identified spatial primitives 
%          W    = identified combination coefficients
%         Tau   = identified delays
%         Rec   = reconstructed data
%          p    = variance accounted for
% 
% Copyright (C) Enrico Chiovetto, 2014, enrico.chiovetto@gmail.com.
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% If you use this software for your research, please refer to the paper
% entitled " ", 
% by Chiovetto E, d'Avella A and Giese, MA.

best_p= -Inf;
if nargin < 6
    error('The function needs more input arguments');
end
if nargin == 6
    NumRuns=5; %Number of runs
end

%---------- The algorithm is run NumRuns times

for k= 1:NumRuns
    disp(['Algorithm run # ',num2str(k),' (out of ',num2str(NumRuns),')']);
    [S_temp, w_temp, W_temp, Tau_temp, Rec_temp, p_temp] = fadaSingleRunNM3F(X, T,...
        NumS, NumW, NumH, del);
    if max(p_temp)>best_p,
        best_p = max(p_temp);
        S = S_temp;
        Ss= w_temp;
        W = W_temp;
        Tau = Tau_temp;
        Rec = Rec_temp;
        p=p_temp;
    end
end

