function [S, W, Rec, p] = fada_spat(X,T,Nsp,M,Spos,Cpos,NumRuns,indep)

% fada_spatial performs Fourier-based demixing of the input data and 
% extracts spatial primitives
%
%  Inputs: X    = Matrix of the original data
%                 X is obtained by transposing the matrix with time signals
%                 of the same trial piled vertically and signals of different
%                 trials concatenated horizontally. X will thus have size
%                 (Ntr x T) x (Ndof). Ntr is the number of trials, Ndof 
%                 the number of degrees of freedom recorded in each trial,
%                 for instance the number of muscles.
%                 In this case the signals of each trial do not need to be
%                 standardized in time.
%
%         Ndof  = Number of degrees of freedom
%         Nsp  = Number of primitives to learn
%         M  = Number of Fourier harmonics to approximate the original
%         signals
%         Spos  = positivity of the primitives 0 = unconstrained
%                                              1 = positive
%         Cpos  = positivity of the combination coefficients
%                                             0 = unconstrained
%                                             1 = positive
%         NumRuns = number of runs
%         indep = independence of the sources  'ica' = independent sources
%                                              'unc' = uncorrelated sources
%                                              'lsq' = Least-square approx.
%
%   Outputs:
%          S    = identified primitives
%          W    = identified combination coefficients
%         Rec   = reconstructed data
%          R2    = variance accounted for
%
% Copyright (C) Enrico Chiovetto, 2014, enrico.chiovetto@gmail.com.
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% If you use this software for your research, please refer to the paper
% entitled " ", 
% by Chiovetto E, d'Avella A and Giese, MA.

best_p = -Inf;
del=0;
if nargin < 6
    error('The function needs more input arguments');
end
if nargin == 6
    NumRuns=5; % Number of runs
    indep='lsq';
end
if nargin == 7
    indep='lsq';
end

%---------- The algorithm is run NumRuns times

for k= 1:1:NumRuns
    disp(['Algorithm run # ',num2str(k),' (out of ',num2str(NumRuns),')']);
    [S_temp, W_temp, ~, Rec_temp, p_temp] = fadaSingleRun(X, T,...
        Nsp, M, Spos, Cpos, del, indep);
    if max(p_temp)>best_p,
        best_p = max(p_temp);
        S = S_temp;
        W = W_temp;
        Rec = Rec_temp;
        p=p_temp;
    end
end

