function[outW, outD, outRec] = weightsDelayEstimation(data,prim,W,D,delay_box,Cpos,del)

% weightsDelayEstimation computes the weights and delays given the
% primitives. Weights and delays are optimized so as to minimize the
% difference between original and reconstructed data.
%
%  Inputs:
%      data     = Matrix of the original data. It has size NumM x T,where
%                 NumM is the number of degrees of freedom recorded in each
%                 trial, for instance the number of muscles. T the number
%                 of time samples).
%
%      prim     = Given primitives
%       W       = Initial values of the weights
%       D       = Initial values of the delays
%   delay_box   = column array containing the upper bound of the delays.
%                 lower bounds are all set to zero. The vector has NumS
%                 elements, where NumS is the number of primitives.
%      Cpos     = positivity of the combination coefficients
%                                             0 = unconstrained
%                                             1 = positive
%      del      = modelling of the delays     0 = non-shiftable primitives
%                                             1 = time shiftable primitives
%
%   Outputs:
%       outW    = updated combination coefficients
%       outD    = updated delays
%      outRec   = reconstructed data
%
% Copyright (C) Enrico Chiovetto, 2014, enrico.chiovetto@gmail.com.
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% If you use this software for your research, please refer to the paper
% entitled " ",
% by Chiovetto E, d'Avella A and Giese, MA.

[NumS,NumM,T]=size(prim);
maxiter = 250;          % maxiter = maximum number of iterations (matching 
                       % pursue procedure)
ifinteger=1;           % 0 = eational delays (even not integer, 1 = integer 
                       % delays
ft_prim = fft(prim,[],3);
binFreq = -2*pi*(mod(((0:T-1)+floor(T/2)), T)-floor(T/2))/T;
best_p = -Inf;

for iter=1:maxiter
    
    %---- Udate the Delays
    
    if del==1
        ind_vect=1:NumS;
        while not(isempty(ind_vect))
            d=zeros(1,numel(ind_vect));
            for k=1:1:numel(ind_vect)
                u = zeros(1,T);
                for l=1:NumM
                    HS = squeezeOne(prim([1:ind_vect(k)-1 ind_vect(k)+1:NumS],l,:));
                    rest = fft(data(l,:)-syn_mix(HS,W(1,[1:ind_vect(k)-1 ind_vect(k)+1:NumS]),D(1,[1:ind_vect(k)-1 ind_vect(k)+1:NumS])),[],2);
                    u           = u + rest .* conj(W(1,ind_vect(k))*permute(squeeze(ft_prim(ind_vect(k),l,:)),[2 1]));
                end
                h    = ifft(u);
                Xcor = abs(h);
                [~,integerDelay]=max(Xcor(1:delay_box(ind_vect(k))));
                integerDelay=integerDelay(1)-1;
                if ifinteger==0,
                    u=u.*exp_vector(-integerDelay,T)';
                    h=abs(u);
                    uDelayPhase=binFreq.*h;
                    ang=atan2(imag(u), real(u)).*h;
                    warning off;
                    fractionalDelay=ang/uDelayPhase;
                    d(1,k)=integerDelay+fractionalDelay;
                else
                    d(1,k)=integerDelay;
                end
            end
            [out1,out2]=max(d);
            D(1,ind_vect(out2))=out1;
            ind_vect(out2)=[];
        end
    else
        D(1,1:NumS)=0;
    end
    
    %---- Update the Weights
    
    aux=zeros(NumS,NumM*T);
    for k=1:1:NumS
        temp=zeros(NumM,T);
        for j=1:1:NumM
            temp(j,:)=syn_mix(squeezeOne(prim(k,j,:)),1,D(1,k));
        end
        aux(k,:)=reshape(temp',1,NumM*T);
    end
    
    options = optimset('Diagnostic','off','Display','off');
    if Cpos==1
        W(1,:) = lsqlin(aux', reshape(data',NumM*T,1),-eye(NumS),zeros(1,NumS),[],[],[],[],[],options);
    else
        W(1,:) = aux'\reshape(data',NumM*T,1);
    end
    
    for k=1:1:NumM
        Rec(k,:)=syn_mix(squeezeOne(prim(:,k,:)),W,D);
    end
    q(iter)=1-(norm(data-Rec,'fro')^2)/(norm(data-repmat(mean(data,2),1,size(data,2)),'fro')^2);
    
    %---------- Update outputs
    if or(q(iter)>best_p,iter==maxiter)
        best_p = q(iter);
        outW = W;
        outD = D;
        outRec = Rec;
    end
    
end